@@ -21,6 +21,10 @@
   USA.
 ***/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <pulsecore/macro.h>
 #include <pulse/sample.h>
 #include <pulse/channelmap.h>
